// Program Nln2.cpp calculats the optimal N within a given
// interval for N, so that N*ln(2) can be stored as a real number
// with a minimal relative error.
// Output:  N, absolute and relative error, approximation of N*ln(2)
//          and optimal inclusion of N*ln(2); 

#include <iostream>     // for cout
#include <l_rmath.hpp>  // for ln(..) with l_interval argument
#include "bnd_util.hpp" // for IEEE2frac_int(..)

using namespace cxsc;
using namespace std;

int main()
{
    int N1,N2,Nopt,N;
    real Nln_2,abs_,minr=MaxReal,mina=MaxReal,rel,z,n,z1,n1,z2,n2;
    stagprec = 4;  // Precision of the staggered arithmetic
    l_interval li_ln2=ln( l_interval(2) ),li_Nln2;
    interval zz;  // For inclusions in the double format
    cout << endl;
    cout << "Calculating an optimal N within an interval [N1,N2] to"
         << endl;
    cout << "store N*ln(2) as one real number with a minimal " 
         << "relative error:" << endl;
    cout << "N1 = ? ";  cin >> N1; 
    cout << "N2 = ? ";  cin >> N2; 
    if (N2<N1) N2 = N1+1;

    N=N1;
    l_real ln2 = ln( l_real(2) ), Nln2;
    while (N<=N2)
    {
        Nln2  = N * ln2;  // Nln2: Staggered approximation of N*ln2
        Nln_2 = Nln2[1];  // Optimal real-approximation for N*ln(2)
        li_Nln2 = N * li_ln2;  // li_Nln2: Inclusion of N*ln(2)
        zz = li_Nln2 - Nln_2;  // zz: Inclusion of  N*ln(2)-Nln_2
        abs_ = Sup(abs(zz)); // abs_: upper bound of the abs. error
        zz = li_Nln2;        // zz: Inclusion of N*ln(2)
        rel = divu(abs_,Inf(abs(zz))); // rel: relative error bound

        if (rel<minr) { 
            minr = rel; mina = abs_; Nopt = N;
            IEEE2frac_int(Nln_2,z,n); 
            // zz: Inclusion of Nopt*ln(2)
            IEEE2frac_int(Inf(zz),z1,n1); // Inf(zz) = z1/n1
            IEEE2frac_int(Sup(zz),z2,n2); // Sup(zz) = z2/n2
        }
        N++;
    }
    cout << "With the optimal N = " << Nopt << " we get:" << endl;
    cout << SetPrecision(14,14) << RndUp 
         << "Optimal relative error < " << minr << endl;
    cout << "Optimal absolute error < " << mina << endl;
    cout << "Optimal approximation of " << Nopt 
         << "*ln(2) with the quotient:"  << endl;
    cout << SetPrecision(1,1) << Fixed << "          " 
         << z << " / " << n << endl;
    cout << "Optimal inclusion of " << Nopt 
         << "*ln(2) with the interval [a,b]:"  << endl;
    cout << "a = " << z1 << " / " << n1 << endl; 
    cout << "b = " << z2 << " / " << n2 << endl << endl;  
}
