// Programm:  Term_relh.cpp zur Abschtzung des relativen
//            Auswertefehlers bei hochgenauer Arithmetik. 
#include "bnd_util.hpp"   // Wegen Term_relh(), Term_Dat(), ...
#include "hilfe.hpp"      // Wegen Cut24()
#include <iostream>       // Wegen cout

using namespace cxsc;
using namespace std;

real Term_1(const interval& Xi)
// Berechnet den relativen Auswertefehler des Terms 
// T(x) = x + 0.5*x^2 + (1/6)*x^3 mit dem Algorithmus: 
// T(x) = x + (y+(q+z)); x=u+v, y = 0.5*u^2, z = 0.5*v*(x+u),
// q=(1/6)*x^3 fr alle Maschinenzahlen des Teilintervalls Xi.
{
    real bnd,del1,del2,del3,del6,eps6,Next;
    interval h1,h2,h3,u6, half=interval(0.5),ui,vi,yi;

    ui = interval(Cut24(Inf(Xi)),Cut24(Sup(Xi)));
    vi = Xi - ui;

    abs_addh1(Xi,0,ui,0,h1,del1);       // x+u
    abs_mulh1(vi,0,h1,del1,h2,del2);    // v*(x+u)
    abs_mulh1(half,0,h2,del2,h1,del1);  // z = 0.5*v*(x+u)

    frac_tools(1,6,u6,Next,del6,eps6);
    // u6: Einschlieung von 1/6;  del6: absolute Fehlerschranke
    abs_mulh1(Xi,0,Xi,0,h2,del2);        // x*x
    abs_mulh1(h2,del2,Xi,0,h3,del3);     // x^3
    abs_mulh1(h1,del1,u6,del6,h2,del2);  // q = (1/6)*x^3

    abs_addh1(h1,del1,h2,del2,h3,del3);  // q+z
    yi = 0.5*ui*ui;  // Einschlieung von y = 0.5*u*u; 
    abs_addh1(yi,0,h3,del3,h1,del1);     // y+(q+z)
    rel_addh1(Xi,0,h1,del1,yi,bnd);      // x + (y+(q+z))
   
    return bnd; // Rckgabe des relativen Auswertefehlers
} // Term_1

real Term_2(const interval& Xi)
// Berechnet den relativen Auswertefehler des Terms 
// T(x) = x + 0.5*x^2 + (1/6)*x^3 mit dem Algorithmus: 
// T(x) = (u+y) + (q+(v+z));  x=u+v, y = 0.5*u^2, z = 0.5*v*(x+u);
// fr alle Maschinenzahlen des Teilintervalls Xi.
{
    real bnd,del1,del2,del3,Next,del6,eps6;
    interval h1,h2,h3,u6, half=interval(0.5),ui,vi,yi;

    ui = interval(Cut24(Inf(Xi)),Cut24(Sup(Xi)));
    vi = Xi - ui;

    abs_addh1(Xi,0,ui,0,h1,del1);       // x+u
    abs_mulh1(vi,0,h1,del1,h2,del2);    // v*(x+u)
    abs_mulh1(half,0,h2,del2,h1,del1);  // z = 0.5*v*(x+u)
    abs_addh1(h1,del1,vi,0,h2,del2);    // v+z

    frac_tools(1,6,u6,Next,del6,eps6);
    // u6: Einschlieung von 1/6;  del6: absolute Fehlerschranke
    abs_mulh1(Xi,0,Xi,0,h1,del1);        // x*x
    abs_mulh1(h1,del1,Xi,0,h3,del3);     // x^3
    abs_mulh1(h3,del3,u6,del6,h1,del1);  // q = (1/6)*x^3
    abs_addh1(h1,del1,h2,del2,h3,del3);  // q+(v+z)

    yi = 0.5*ui*ui;  // Einschlieung von y = 0.5*u*u;
    h1 = yi+ui; // Einschlieung fr:  y+u
    del1 = 0;   // u+0.5*u^2=u+y wird bez. CuT24() rundungsfehlerfrei
                // addiert fr x aus [ 2^(-4),2^(+30.5) ) 
    rel_addh1(h1,del1,h3,del3,yi,bnd);
    
    return bnd; // Rckgabe des relativen Auswertefehlers
}

int main()
{
// Berechnung einer relativen Fehlerschranke bez. der Auswertung
// eines Terms bei hochgenauer Arithmetik.

    interval X;  // Intervall der Maschinen-Argumente des Terms
    real bnd,diam;

    while(1) // Endlosschleife
    {
        Term_Dat(X,diam);  // Eingabe von X und diam;
	    // X   : Einschlieung der Maschinen-Argumente des Terms
            // diam: Mantissendurchmesser der Teilintervalle (10^(-5))
        bnd = Term_relh(Term_2,X,diam); // bnd: relative Fehlerschranke
        // bei nur hochgenauer Arithmetik bei der Termauswertung.
        cout << RndUp 
             << "Relative Fehlerschranke bei hochgenauer Arithmetik = "
	     << bnd << endl << endl; 
    }
}


















