/*
**  CXSC is a C++ library for eXtended Scientific Computing
**
**  Copyright (C) 1990-2000 Institut fuer Angewandte Mathematik,
**                          Universitaet Karlsruhe, Germany
**            (C) 2000-2001 Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
**  In dieser Datei werden nur solche Funktionen deklariert, die fr den
**  normalen Anwender nicht erreichbar sein sollen.
**  Die folgenden Funktionen   uint_trail(...) und sqr2uv(...)   sind in
**  rmath.cpp  implementiert. 
**  Will man diese Funktionen benutzen, so muss man im Programm
**  #include "hilfe.hpp"   oben einfgen;  Blomquist, 06.11.2005; 
**  Zur Sicherheit ist  hilfe.cpp  in /home/blom/cxsc-Werkstatt  zu finden.    
*/

#ifndef hilfe_hpp_included
#define hilfe_hpp_included

#include <real.hpp>

namespace cxsc
{
    int uint_trail(const unsigned int& n);
    //  Liefert die Anzahl aufeinanderfolgender Null-Bits am Ende von n;
    //  n=0 --> 32;   n=1 --> 0;   n=2 --> 1;   n=4 --> 2;   n=1024 --> 10;

    void sqr2uv(const real& x, real& u, real& v);
    // Liefert u,v fr: x^2 = u + v; EXAKTE Darstellung, falls kein overflow 
    // auftritt und v im normalisierten Bereich liegt. u > |v|
    // Vorsicht: Funktioniert zunchst nur auf INTEL-Systemen!!!

} // namespace cxsc 

#endif







