// Programm sqrt1mx2.cpp zur Auswertung der Funktion
// sqrt(1-x^2) fr Punktargumente x aus [-1,+1]
// und fr Intervallargumente. 

#include <rmath.hpp>  // Wegen sqrt()
#include <imath.hpp>
#include <iostream>

using namespace std;
using namespace cxsc;

real Sqrt1mx2(const real& x) throw(STD_FKT_OUT_OF_DEF)
// sqrt(1-x^2);  rel. Fehlerschranke:  eps = 3.700747E-16 = e(f)
{ 
    real t=x,res;
    int ex;
    if (sign(t)<0) t = -t; // Argument t >=0;
    if (t>1) cxscthrow(STD_FKT_OUT_OF_DEF
                                     ("real sqrt1mx2(const real&)"));
    // Fuer t gilt jetzt:  0 <= t <=1;
    ex = expo(t);
    if (ex<=-26) res = 1; // t < 2^(-26) --> res = 1
    else if (ex<=-15) {   // t < 2^(-15) --> res = 1-x^2/2
        res = t*t;  
        times2pown(res,-1);
        res = 1 - res;
    } else {
        if (ex>=0) 
        {  // ex>=0 --> t>=0.5; 
            res = 1-t;   // res: delta = 1-t;
            t = res * res;
            times2pown(res,1); // res: 2*delta
            res = res - t;     // res: 1-x^2 = 2*delta - delta^2
        } else res = 1-t*t;    // res: Maschinenwert von 1-x^2
        res = sqrt(res);       // res: Nullte Naeherung 
    }
    return res;
} // Sqrt1mx2

// Konstanten fr die Intervallfunktion sqrt(1-x^2):
real q_sqrt1mx2p(4503599627370501.0/4503599627370496.0);  // (1+e(f))
real q_sqrt1mx2m(9007199254740985.0/9007199254740992.0);  // (1-e(f))

interval Sqrt1mx2(const interval& x)
// sqrt(1-x^2);  Blomquist, 13.04.04;
{
    interval z = abs(x);  // sqrt(1-t^2) monoton fallend in t aus z;
    real r1,r2,sz,iz;
    sz = Sup(z);  iz = Inf(z);
    // Berechnung der Unterschranke r1:
    r2 = sqrt1mx2(sz);
    r1 = (sz==0)? 1 : r2 * q_sqrt1mx2m;
    // Berechnung der Oberrschranke r2:
    if (iz<4.81e-8) r2 = 1; // r2=1 ist immer korrekte Oberschranke!
    else r2 = (sz==iz)? r2*q_sqrt1mx2p : sqrt1mx2(iz)*q_sqrt1mx2p;
    return interval(r1,r2);

} // Sqrtx2m1 (Intervallargumente)

int main()
{
    interval x,y;
    while (1) {
        cout << "interval x = ?" << endl;   cin >> x;
        y = Sqrt1mx2(x);
        cout << SetPrecision(15,15) 
             << Scientific << "Sqrt1mx2(x) = " << y << endl << endl;
    };
}
