// Programm:  sqrt1mx2_bnd.cpp;
// Berechnung der rel. Fehlerschranke eps(f) in 
// 0 <= x <= pred(1.0) fuer sqrt( 1-x^2 );

#include "abs_relh.hpp" // Wegen abs_mulh1(),...
#include "bnd_util.hpp" // Wegen Max_bnd_Xi()
#include <iostream>     // Wegen cout

using namespace cxsc;
using namespace std;

real T_xA2(const interval& x, const real& delx)
{   // x: Teilintervall von A2
    interval X2,R,z,zz;
    real delx2,ra,rel;

    abs_mulh1(x,delx,x,delx,X2,delx2);          // X2 = x*x
    abs_divh1(X2,delx2,interval(2),0.0,R,rel);  // R  = (x*x)/2
    rel_subh1(interval(1.0),0,R,rel,z,ra);      // z  = 1-(x*x)/2
    // ra: relativer Auswertefehler;
    z = Sup(X2);  zz = z*z;
    z = zz / (8*(1-z));  // Sup(z): relativer Approximationsfehler
    // Berechnung einer Schranke des relativen Gesamtfehlers:
    z = Sup(z);   zz = ra;
    z = z + zz*(1+z);
    rel = Sup(z);
    return rel;  // Schranke des relativen Gesamtfehlers.
}

real T_xA3(const interval& x, const real& delx)
{   // x: Teilintervall von A3
    interval X2,Az,R;
    real delx2,dela,rel;

    abs_mulh1(x,delx,x,delx,X2,delx2);  // X2 = x*x
    abs_subh1(interval(1.0),0,X2,delx2,Az,dela); // Az = 1-x^2
    rel_sqrt_abs(Az,dela,R,rel);

    return rel;  // Schranke des relativen Gesamtfehlers.
}

real T_xA4(const interval& Di, const real& delx)
{   // Di: Teilintervall von D
    interval G,H,C;
    real delh,delc,bnd;
    G=Di;
    times2pown(G,1);  // G = 2*Di
    abs_mulh1(Di,delx,Di,delx,H,delh);  // H = Di*Di
    abs_subh1(G,0,H,delh,C,delc);  // C = 2*Di - Di*Di
                  // delc: abs. Fehlerschranke bez. 2*Di - Di*Di
    rel_sqrt_abs(C,delc,G,bnd);    // G = sqrt(2*Di - Di*Di)
    return bnd;
}

int main()
{   
    interval A1 = interval(0,comp(0.5,-25)),
             A2 = interval(comp(0.5,-25),comp(0.5,-14)),
             A3 = interval(0.1,0.5), 
             A4 = interval(0.5,pred(1.0)),
             D  = 1 - A4,z,zz;
    real bnd1,bnd2,bnd3,bnd4,bnd=0,
         diam = 1e-6, delx=0;
    z = A1*A1;  z = Sup(z);
    z = z / (2*(1-z));
    bnd1 = Sup(z);
    if (bnd1>bnd) bnd = bnd1; 
    cout << RndUp << "Rel. Schranke eps(f,A1) fuer A1  = " << bnd1 
         << endl;
    Max_bnd_Xi(T_xA2,A2,delx,diam,bnd2);
    cout << "Rel. Schranke eps(f,A2) fuer A2  = " << bnd2 << endl;
    if (bnd2>bnd) bnd = bnd2;
    Max_bnd_Xi(T_xA3,A3,delx,diam,bnd3);
    cout << "Rel. Schranke eps(f,A3) fuer A3  = " << bnd3 << endl;
    if (bnd3>bnd) bnd = bnd3;
    Max_bnd_Xi(T_xA4,D,delx,diam,bnd4);
    cout << "Rel. Schranke eps(f,A4) fuer A4  = " << bnd4 << endl;
    if (bnd4>bnd) bnd = bnd4; 
    cout << "Rel. Gesamtfehlerschranke eps(f) = " << bnd << endl;
}
