// Programm sqrtx2m1.cpp zur Auswertung der Funktion
// sqrt(x^2-1) fr Punktargumente x aus [1,MaxReal]
// und fr Intervallargumente.
#include <rmath.hpp>  // Wegen sqrt()
#include <interval.hpp>
#include "hilfe.hpp"  // Wegen Cut26()
#include <iostream>

using namespace std;
using namespace cxsc;

real q_sqrtx2m1p(4503599627370501.0/4503599627370496.0);  // (1+e(f))
real q_sqrtx2m1m(9007199254740986.0/9007199254740992.0);  // (1-e(f))

real Sqrtx2m1(const real& x)
{  // sqrt(x^2-1);  rel. Fehlerschranke:  eps = 2.221305E-16
    const real c1 = 1.000732421875, 
               c2 = 44000.0,
               c3 = 1024.0;  // c1,c2,c3 werden exakt gespeichert!
    real res,ep,ep2,s1,s2,x1,x2,arg=x;
    if (sign(arg)<0) arg = -arg; // arg = |x| >= 0
    if (arg <= c1) { // x = 1+ep; x^2-1 = 2*ep + ep^2;
        ep = x - 1; // Differenz rundungsfehlerfrei!
        ep2 = ep*ep;  // ep2 i.a. fehlerbehaftet!
        times2pown(ep,1);  // ep = 2*ep;
        res = sqrt(ep+ep2); // res=y0: Startwert;
        // x - y0^2 = (2*eps - s1^2) + [eps^2 - s2*(y0 + s1)]
        s1 = Cut26(res);  s2 = res - s1; // Startwert = s1 + s2;
        arg = ep - s1*s1;  // arg = 2*eps - s1^2;
        arg += (ep2 - s2*(res+s1));  // arg = x - y0^2
        if (sign(arg)>0) {
        arg = arg / res;
        times2pown(arg,-1);
        res += arg;  // 1. Newton-Schritt beendet; eps = 2.221261E-16
        }
    } else if (arg<c2) { 
        // x-y0^2 = [(x1^2-1)-s1^2] + [x2*(x+x1)-s2*(y0+s1)]
        x1 = Cut26(arg);  x2 = arg - x1;  // arg = x = x1 + x2;
        ep2 = x2*(arg+x1);  // ep2 ist fehlerbehaftet
        x2 = x1*x1;  ep = x2-1;
        res = sqrt(ep+ep2); // res ist Startwert fr Newton-Verfahren
        s1 = Cut26(res);  s2 = res - s1;  // Startwert = s1 + s2;
        ep2 = ep2 - s2 * (res+s1); // ep2 = [x2*(x+x1)-s2*(y0+s1)]
        if (arg<c3) ep -= s1*s1;   // ep = (x1^2-1) - s1^2;
        else {
            x2 -= s1*s1;  // x2 = x1^2-s1^2
            ep = x2 - 1; }         // ep = (x1^2-s1^2) - 1;
        ep += ep2;        // ep = x - y0^2;
        ep /= res;
        times2pown(ep,-1);
        res = res + ep;  // 1. Newton-Schritt in hoher Genauigkeit
                         // beendet;  eps = 2.221305E-16
    } else { // arg = |x| >= 44000;
        res = -1/arg;
        times2pown(res,-1); // Multiplikation mit 0.5;
        res += arg;  // res = x - 1/(2*x);  eps = 2.221114E-16
    }
    return res;
} // Sqrtx2m1 (Punktargumente)

interval Sqrtx2m1(const interval& x)
{
    interval z = abs(x);
    real r1,r2;
    r1 = sqrtx2m1(Inf(z)) * q_sqrtx2m1m;
    r2 = Sup(z); 
    if (expo(r2)<26)  
        r2 = sqrtx2m1(r2) * q_sqrtx2m1p; 
    // expo(r2) >= 26 --> r2 = Sup(z) ist die optimale Oberschranke!
    return interval(r1,r2);
} // Sqrtx2m1 (Intervallargumente)

int main()
{
    interval x,y;

    while (1) {
        cout << "interval x = ?" << endl;
        cin >> x;
        y = Sqrtx2m1(x);
        cout << SetPrecision(20,20) << Scientific << "Sqrtx2m1(x) = " 
             << y << endl << endl;
    };
}
