// Programm:  sqrtx2m1_A1.cpp;
// Berechnung der rel. Fehlerschranke eps(f) in A1:
// x = 1 + DEL;  --->  x^2-1 = 2*DEL + DEL^2; 
// y0 = s1 + s2; y0 = sqrt(x^2-1) = sqrt(2*DEL + DEL^2) ist Startwert 
// x - y0^2 = (2*DEL - s1^2) + [DEL^2 - s2*(y0 + s1)]
#include "abs_relh.hpp" // Wegen abs_mulh1()
#include "bnd_util.hpp" // Wegen Max_bnd_Xi()
#include "hilfe.hpp"    // Wegen CuT26()
#include <iostream>     // Wegen cout
using namespace cxsc;
using namespace std;
real T_x(const interval& Di, const real& delx)
{   // Di: Teilintervall von DEL
    interval Y0,B,C,D,E,S1,S2,G,H;
    real s1,s2,rel,app,delb,delc,dele,delh;
    int ex;

    G=Di;
    times2pown(G,1);  // G = 2*Di
    abs_mulh1(Di,delx,Di,delx,H,delh);  // H = Di*Di
    abs_addh1(G,0,H,delh,C,delc);  // C = 2*Di + Di*Di
                  // delc: abs. Fehlerschranke bez. 2*Di + Di*Di
    rel_sqrt_abs(C,delc,Y0,app);  // Y0 = sqrt(2*Di + Di*Di)
    // app: relative Fehlerschranke fuer den Startwert.
    D = interval(-app,+app);
    D = 0.5 * D*D/(1+D);
    app = Sup(D);  // app: Schranke des rel. Approximationsfehlers
    
    s1 = Cut26(Inf(Y0));  s2 = Cut26(Sup(Y0));
    S1 = interval(s1,s2); // y0 = s1+s2; s1 = Cut26(y0)
    ex = expo(Sup(Y0));
    s1 = comp(0.5,ex-25);
    S2 = interval(0,s1);  // s1 in S1 und s2 in S2 enthalten!!

    B = S1*S1;  // s1*s1 wird stets rundungsfehlerfrei berechnet!
    abs_subh1(G,0.0,B,0.0,D,s1); // D = 2*Di-S1*S1; s1=0 erfuellt
    abs_addh1(Y0,0.0,S1,0.0,B,delb); // B = y0 + s1
    abs_mulh1(B,delb,S2,0.0,C,delc); // C = s2*(y0 + s1)
    abs_subh1(H,delh,C,delc,B,delb); // B = Di*Di - s2*(y0 + s1)
    abs_addh1(D,s1,B,delb,E,dele);   // E = (2Di + Di*Di) - y0^2
    abs_divh1(E,dele,Y0,0.0,C,delc); // C = [(2Di+Di*Di)-y0^2]/y0
    C = C/2;  delc = delc/2;
    rel_addh1(Y0,0.0,C,delc,B,rel);   // rel: rel. Auswertefehler

    // Beruecksichtigung des relativen Approximationsfehlers:
    s1 = addu(1.0,app);
    s2 = mulu(s1,rel);
    rel = addu(app,s2);

    return rel;  // Schranke des relativen Gesamtfehlers.
}

int main()
{   // 1.000732421875 = 1 + 3*2^(-12) wird exakt gespeichert!
    interval X = interval(succ(1.0),1.000732421875),DEL;
    DEL = X - 1;
    real bnd, diam = 1e-6, delx=0;
    Max_bnd_Xi(T_x,DEL,delx,diam,bnd);
    cout << RndUp << "Rel. Schranke eps(f) = " << bnd << endl;
}
